/*
Module : DYNDATA.CPP
Purpose: Defines the implementation for a class to access the performance data as stored under the HKEY_DYN_DATA registry 
         key on Windows 9x. Please note that this class is not designed to be used on NT, 2000, XP, 2003, Vista.
Created: PJN / 31-01-1999
History: PJN / 10-01-2007 1. Updated copyright details.
                          2. Addition of a DYNDATA_EXT_CLASS preprocessor macro to allow the classes to be more easily added to an 
                          extension dll
                          3. Optimized CDynDataObject constructor code.
                          4. Reviewed all TRACE statements for correctness
                          5. Code now uses newer C++ style casts instead of C style casts.
                          6. Item data value passed to the various enumeration functions is now a DWORD_PTR instead of a DWORD
                          7. ENUMERATE_DYNDATA_OBJECT_COUNTERS is now declared in the CDynDataObject class scope
                          8. ENUMERATE_DYNDATA_OBJECTS is now declared in the CDynDataEnumerator class scope
                          9. Updated the code to clean compile on VC 2005
                          10. Fixed a bug in the calculation of the buffer sizes passed to RegQueryValueEx in CDynDataObject::_EnumCounters
                          11. General code cleanup and code review
                          12. Updated documentation to use the same style as the web site

Copyright (c) 1999 - 2007 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

///////////////////////////// Includes ////////////////////////////////////////

#include "stdafx.h"
#include "dyndata.h"


///////////////////////////// Statics / Locals ////////////////////////////////

#ifdef _DEBUG
#undef THIS_FILE
#define new DEBUG_NEW
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
             

///////////////////////////// Implementation //////////////////////////////////

CDynDataObject::CDynDataObject(LPCTSTR pszObjName, HKEY hObjectKey) : m_sName(pszObjName),
                                                                      m_hObjectKey(hObjectKey)  
{
  //Validate our parameters
  ASSERT(m_sName.GetLength());
  ASSERT(hObjectKey);

  //Hive away the Object description
  TCHAR szDescription[_MAX_PATH];
  DWORD dwType;
  DWORD dwBufSize = _MAX_PATH*sizeof(TCHAR);
  DWORD dwConnect = RegQueryValueEx(hObjectKey, _T("Name"), NULL, &dwType, reinterpret_cast<LPBYTE>(szDescription), &dwBufSize);
  if (dwConnect == ERROR_SUCCESS)
    m_sDescription = szDescription;
  else
    TRACE(_T("CDynDataObject::CDynDataObject, Failed to get object description, Return code was %d"), dwConnect);
}

CDynDataObject::CDynDataObject(LPCTSTR pszObjName, LPCTSTR pszComputerName) : m_hObjectKey(NULL),
                                                                              m_sName(pszObjName) 
{
  //Store away the values
  if (pszComputerName)
    m_sComputerName = pszComputerName;

  //validate our parameters
  ASSERT(m_sName.GetLength());

  //Cannot hive away the description until EnumCounters is called
}

BOOL CDynDataObject::_EnumCounters(ENUMERATE_DYNDATA_OBJECT_COUNTERS lpEnumFunc, DWORD_PTR dwItemData, HKEY hObjectKey)
{
  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;
  
  DWORD dwIndex = 0;
  TCHAR szObjectName[_MAX_PATH];
  DWORD dwBufSize = _MAX_PATH;
  DWORD dwConnect = RegEnumKeyEx(hObjectKey, dwIndex, szObjectName, &dwBufSize, NULL, NULL, NULL, NULL);
  BOOL bContinue = TRUE;
  while ((dwConnect == ERROR_SUCCESS) && bContinue)
  {
    //We've enumerated at least one object, so the call is successful.
    bSuccess = TRUE;

    //Call the enum function with an object setup
    HKEY hCounter = NULL;
    dwConnect = RegOpenKeyEx(hObjectKey, szObjectName, 0, KEY_READ, &hCounter);
    if (dwConnect == ERROR_SUCCESS)
    {
      //Get the name of the counter
      TCHAR szCounterName[_MAX_PATH];
      szCounterName[0] = _T('\0');
      DWORD dwType;
      DWORD dwBufSize = _MAX_PATH*sizeof(TCHAR);
      DWORD dwConnect = RegQueryValueEx(hCounter, _T("Name"), NULL, &dwType, reinterpret_cast<LPBYTE>(szCounterName), &dwBufSize);
      if (dwConnect != ERROR_SUCCESS)
        TRACE(_T("CDynDataObject::_EnumCounters, Failed to get counter name, Return code was %d"), dwConnect);

      //Get the description of the counter
      TCHAR szDescription[_MAX_PATH];
      szDescription[0] = _T('\0');
      dwBufSize = _MAX_PATH*sizeof(TCHAR);
      dwConnect = RegQueryValueEx(hCounter, _T("Description"), NULL, &dwType, reinterpret_cast<LPBYTE>(szDescription), &dwBufSize);
      if (dwConnect != ERROR_SUCCESS)
        TRACE(_T("CDynDataObject::_EnumCounters, Failed to get counter description, Return code was %d"), dwConnect);

      //Get the differentiate flag of the counter
      TCHAR szDifferentiate[_MAX_PATH];
      dwBufSize = _MAX_PATH*sizeof(TCHAR);
      BOOL bDifferentiate = FALSE;
      dwConnect = RegQueryValueEx(hCounter, _T("Differentiate"), NULL, &dwType, reinterpret_cast<LPBYTE>(szDifferentiate), &dwBufSize);
      if (dwConnect != ERROR_SUCCESS)
        TRACE(_T("CDynDataObject::_EnumCounters, Failed to get counter differentiate flag, Return code was %d"), dwConnect);
      else
      {
        if (_tcscmp(szDifferentiate, _T("TRUE")) == 0)
          bDifferentiate = TRUE;
      }

      //Call the callback function
      bContinue = lpEnumFunc(dwItemData, *this, szObjectName, szCounterName, szDescription, bDifferentiate);
    }
    else
      TRACE(_T("CDynDataObject::_EnumCounters, Failed in call to open counter key, Return code was %d\n"), dwConnect);

    //Don't forget to close the key
    RegCloseKey(hCounter);

    if (bContinue)
    {
      //Set up for the next loop around
      dwIndex++;
      dwBufSize = _MAX_PATH;
      dwConnect = RegEnumKeyEx(hObjectKey, dwIndex, szObjectName, &dwBufSize, NULL, NULL, NULL, NULL);
    }
  }

  return bSuccess;
}

BOOL CDynDataObject::EnumCounters(ENUMERATE_DYNDATA_OBJECT_COUNTERS lpEnumFunc, DWORD_PTR dwItemData)
{
  //Validate our parameters
  ASSERT(lpEnumFunc != NULL);
  ASSERT(m_sName.GetLength());

  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //If the object registry key is not open then open it and call the internal enum function
  //with the registry key obtained
  if (m_hObjectKey == NULL)
  {
    //Open up the HKEY_LOCAL_MACHINE key on the specified macine
    HKEY hKeyLocalMachine = NULL;
    DWORD dwConnect = RegConnectRegistry(m_sComputerName.GetLength() ? m_sComputerName.operator LPCTSTR() : NULL, HKEY_LOCAL_MACHINE, &hKeyLocalMachine);
    if (dwConnect == ERROR_SUCCESS)
    {
      //Open up the "System\CurrentControlSet\Control\PerfStats\Enum\'m_sName'" key
      CString sRegKey;
      sRegKey.Format(_T("System\\CurrentControlSet\\Control\\PerfStats\\Enum\\%s"), m_sName.operator LPCTSTR());
      HKEY hObjectKey = NULL;
      dwConnect = RegOpenKeyEx(hKeyLocalMachine, sRegKey, 0, KEY_READ, &hObjectKey);
      if (dwConnect == ERROR_SUCCESS)
      {
        //Hive away the Object description
        TCHAR szDescription[_MAX_PATH];
        DWORD dwType;
        DWORD dwBufSize = _MAX_PATH*sizeof(TCHAR);
        DWORD dwConnect = RegQueryValueEx(hObjectKey, _T("Name"), NULL, &dwType, reinterpret_cast<LPBYTE>(szDescription), &dwBufSize);
        if (dwConnect == ERROR_SUCCESS)
          m_sDescription = szDescription;
        else
        {
          TRACE(_T("CDynDataObject::EnumCounters, Failed to get object description, Return code was %d"), dwConnect);
          m_sDescription.Empty();
        }

        //Call the internal function to do the enumeration
        bSuccess = _EnumCounters(lpEnumFunc, dwItemData, hObjectKey);

        //Don't forget to close our key
        RegCloseKey(hObjectKey);
      }
      
      //Don't forget to close our key
      RegCloseKey(hKeyLocalMachine);
    }
  }
  else
    bSuccess = _EnumCounters(lpEnumFunc, dwItemData, m_hObjectKey);
  
  return bSuccess;
}


BOOL CDynDataEnumerator::EnumObjects(ENUMERATE_DYNDATA_OBJECTS lpEnumFunc, DWORD_PTR dwItemData, LPCTSTR pszComputerName)
{
  //Validate out parameters
  ASSERT(lpEnumFunc);

  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //Open up the HKEY_LOCAL_MACHINE key on the specified macine
  HKEY hKeyLocalMachine = NULL;
  DWORD dwConnect = RegConnectRegistry(pszComputerName, HKEY_LOCAL_MACHINE, &hKeyLocalMachine);
  if (dwConnect == ERROR_SUCCESS)
  {
    //Open up the "System\CurrentControlSet\Control\PerfStats\Enum" key
    HKEY hEnumKey = NULL;
    dwConnect = RegOpenKeyEx(hKeyLocalMachine, _T("System\\CurrentControlSet\\Control\\PerfStats\\Enum"), 0, KEY_READ, &hEnumKey);
    if (dwConnect == ERROR_SUCCESS)
    {
      //Enumerate all the keys under the Enum key
      DWORD dwIndex = 0;
      TCHAR szObjectName[_MAX_PATH];
      DWORD dwBufSize = _MAX_PATH;
      dwConnect = RegEnumKeyEx(hEnumKey, dwIndex, szObjectName, &dwBufSize, NULL, NULL, NULL, NULL);
      BOOL bContinue = TRUE;
      while ((dwConnect == ERROR_SUCCESS) && bContinue)
      {
        //We've enumerated at least one object, so the call is successful.
        bSuccess = TRUE;

        //Call the enum function with an object setup
        HKEY hObject = NULL;
        dwConnect = RegOpenKeyEx(hEnumKey, szObjectName, 0, KEY_READ, &hObject);
        if (dwConnect == ERROR_SUCCESS)
        {
          CDynDataObject object(szObjectName, hObject);
          bContinue = lpEnumFunc(dwItemData, object);
        }
        else
          TRACE(_T("CDynDataEnumerator::EnumObjects. Failed in call to open object key, Return code was %d\n"), dwConnect);

        //Don't forget to close the key
        RegCloseKey(hObject);

        if (bContinue)
        {
          //Set up for the next loop around
          dwIndex++;
          dwBufSize = _MAX_PATH;
          dwConnect = RegEnumKeyEx(hEnumKey, dwIndex, szObjectName, &dwBufSize, NULL, NULL, NULL, NULL);
        }
      }

      //Don't forget to close our key
      RegCloseKey(hEnumKey);
    }
    else
      TRACE(_T("CDynDataEnumerator::EnumObjects, Failed to open the PerfStats\\Enum key, Return code was %d\n"), dwConnect);

    //Don't forget to close our key
    RegCloseKey(hKeyLocalMachine);
  } 
  else
  {
    TRACE(_T("CDynDataEnumerator::EnumObjects, Failed to open the HKEY_LOCAL_MACHINE key, Return code was %d\n"), dwConnect);
    SetLastError(dwConnect);
  }

  return bSuccess;
}


CDynDataCounter::CDynDataCounter()
{
}

CDynDataCounter::~CDynDataCounter()
{
  Stop();
}

BOOL CDynDataCounter::Start(LPCTSTR pszObjName, LPCTSTR pszCounterName, LPCTSTR pszComputerName)
{
  //Validate our parameters
  ASSERT(pszObjName);
  ASSERT(pszCounterName);

  //Stop any previous collection
  Stop();

  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //Open up the HKEY_DYN_DATA key on the specified machine
  HKEY hKeyDynData = NULL;
  DWORD dwConnect = RegConnectRegistry(pszComputerName, HKEY_DYN_DATA, &hKeyDynData);
  if (dwConnect == ERROR_SUCCESS)
  {
    //Open the "HKEY_DYN_DATA\PerfStats\StartStat" registry key
    HKEY hOpen = NULL;
    if (RegOpenKeyEx(hKeyDynData, _T("PerfStats\\StartStat"), 0, KEY_READ, &hOpen) == ERROR_SUCCESS)
    {
      //Form the counter name which we are querying
      CString sCounterName;
      sCounterName.Format(_T("%s\\%s"), pszObjName, pszCounterName);
      
      //Query to get data size
      DWORD dwType = 0;
      DWORD cbData = 0;
      if (RegQueryValueEx(hOpen, sCounterName, NULL, &dwType, NULL, &cbData) == ERROR_SUCCESS)
      {
        //Query the performance start key to initialise the performace data
        BYTE* pByte = new BYTE[cbData];
        bSuccess = (RegQueryValueEx(hOpen, sCounterName, NULL, &dwType, pByte, &cbData) == ERROR_SUCCESS);

        //Remember the name of the computer we want
        if (pszComputerName)
          m_sComputerName = pszComputerName;
        else
          m_sComputerName.Empty();

        //Remember the name of the counter we want
        m_sCounterName = sCounterName;

        //Don't forget to delete the bit of memory we allocated
        delete [] pByte;
      }

      //Don't forget to close the registry key
      RegCloseKey(hOpen);
    }
    else
      TRACE(_T("CDynDataCounter::Start, Failed to open the PerfStats\\StartStat key, Return code was %d\n"), dwConnect);
  
    //Don't forget to close the registry key
    RegCloseKey(hKeyDynData);
  }
  else
  {
    TRACE(_T("CDynDataCounter::Start, Failed to open the HKEY_DYN_DATA key, Return code was %d\n"), dwConnect);
    SetLastError(dwConnect);
  }

  return bSuccess;
}

BOOL CDynDataCounter::Collect(DWORD& dwData)
{
  //Validate our parameters
  ASSERT(m_sCounterName.GetLength()); //did you forget to call Start

  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //Open up the HKEY_DYN_DATA key on the specified machine
  HKEY hKeyDynData = NULL;
  DWORD dwConnect = RegConnectRegistry(m_sComputerName, HKEY_DYN_DATA, &hKeyDynData);
  if (dwConnect == ERROR_SUCCESS)
  {
    //Open up the "HKEY_DYN_DATA\PerfStats\StatData" key
    HKEY hOpen = NULL;
    if (RegOpenKeyEx(hKeyDynData, _T("PerfStats\\StatData"), 0, KEY_READ, &hOpen) == ERROR_SUCCESS)
    {
      //Query the specifed key for the data
      DWORD dwType = 0;
      DWORD cbData = sizeof(dwData);
      bSuccess = (RegQueryValueEx(hOpen, m_sCounterName, NULL, &dwType, reinterpret_cast<LPBYTE>(&dwData), &cbData) == ERROR_SUCCESS);

      //Don't forget to close the registry key
      RegCloseKey(hOpen);
    }
    else
      TRACE(_T("CDynDataCounter::Collect, Failed to open the PerfStats\\StatData key, Return code was %d\n"), dwConnect);
  }
  else
  {
    TRACE(_T("CDynDataCounter::Collect, Failed to open the HKEY_DYN_DATA key, Return code was %d\n"), dwConnect);
    SetLastError(dwConnect);
  }
  
  return bSuccess;
}

BOOL CDynDataCounter::Stop()
{
  //Handle calling StopCollecting twice
  if (m_sCounterName.IsEmpty())
    return TRUE;

  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //Open up the HKEY_DYN_DATA key on the specified machine
  HKEY hKeyDynData = NULL;
  DWORD dwConnect = RegConnectRegistry(m_sComputerName.GetLength() ? m_sComputerName.operator LPCTSTR() : NULL, HKEY_DYN_DATA, &hKeyDynData);
  if (dwConnect == ERROR_SUCCESS)
  {
    //Open the "HKEY_DYN_DATA\PerfStats\StopStat" registry key
    HKEY hOpen = NULL;
    if (RegOpenKeyEx(hKeyDynData, _T("PerfStats\\StopStat"), 0, KEY_READ, &hOpen) == ERROR_SUCCESS)
    {
      //Query to get data size
      DWORD dwType = 0;
      DWORD cbData = 0;
      if (RegQueryValueEx(hOpen, m_sCounterName, NULL, &dwType, NULL, &cbData) == ERROR_SUCCESS)
      {
        //Query the performance stop key to disable collection of performace data
        BYTE* pByte = new BYTE[cbData];
        bSuccess = (RegQueryValueEx(hOpen, m_sCounterName, NULL, &dwType, pByte, &cbData) == ERROR_SUCCESS);
        delete [] pByte;
      }
      else
        TRACE(_T("CDynDataCounter::Stop, Failed to open the PerfStats\\StopData key, Return code was %d\n"), dwConnect);

      //Don't forget to close the registry key
      RegCloseKey(hOpen);
    }
    

    //Don't forget to close the registry key
    RegCloseKey(hKeyDynData);
  }
  else
  {
    TRACE(_T("CDynDataCounter::Stop, Failed to open the HKEY_DYN_DATA key, Return code was %d\n"), dwConnect);
    SetLastError(dwConnect);
  }

  //Empty out the name of the counter we are collecting
  m_sCounterName.Empty();

  return bSuccess;
}
